;; Copyright 2009, 2010, 2019 Kevin Ryde.

;; This file is part of tcd-format.
;;
;; tcd-format is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; tcd-format is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with tcd-format; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))
(require 'tcd-format)

(defun my-restore-tide-db-available-p ()
  "Return non-nil if the \"restore_tide_db\" program is available."
  ;; any exit status, just not an error throw for no such program; as of
  ;; tcd-utils 20061127 there's no apparent --version argument supported
  (condition-case nil
      (progn
        (let ((default-directory "/")
              (process-connection-type nil)) ;; pipe
          (call-process "restore_tide_db"
                        nil  ;; stdin
                        nil  ;; stdout
                        nil  ;; display
                        "--version"))
        t)
    (error nil)))

(defun my-run-test ()
  (let ((filename "/usr/share/xtide/harmonics-initial.tcd")
        (count 0))

    ;;-------------------------------------------------------------------------
    ;; check autoloads

    ;; emacs24 `generate-file-autoloads' wants to be in a file-visiting
    ;; buffer
    (require 'autoload)
    (find-file "t-xtide--generated-loaddefs.el")
    (erase-buffer)
    (generate-file-autoloads "../tcd-format.el")
    (eval-buffer)
    (save-buffer)
    (kill-buffer (current-buffer))
    (delete-file "t-xtide--generated-loaddefs.el")

    (message "autoloads of decodes")
    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(tcd-format-decode
              tcd-format-encode))

    (mapcar (lambda (symbol)
              (unless (memq t
                            (mapcar (lambda (elem)
                                      (and (eq symbol (car-safe elem))
                                           (progn
                                             (mapcar (lambda (func)
                                                       (unless (fboundp func)
                                                         (error "`%S' not autoloaded" func)))
                                                     (list (nth 3 elem)
                                                           (nth 4 elem)))
                                             t)))
                                    format-alist))
                (error "`%S' not autoloaded in `format-alist'" symbol))
              (setq count (1+ count)))
            '(tcd))

    ;;-------------------------------------------------------------------------

    (message "check on %S" filename)
    (if (not (my-restore-tide-db-available-p))
        (message "  skipped, restore_tide_db program not available")

      (if (not (file-exists-p filename))
          (message "  skipped, no %S file available" filename)

        (progn
          (find-file filename)
          (or (memq 'tcd buffer-file-format)
              (error "`tcd' not in `buffer-file-format' for %S"
                     filename))
          (kill-buffer nil)
          (setq count (1+ count)))

        ;;---------------------------------------------------------------------
        ;; decoding when non-existent default-directory

        (progn
          (switch-to-buffer "*t-tcd-format.el test*")
          (if (fboundp 'set-buffer-multibyte) ;; not in xemacs
              (set-buffer-multibyte nil))
          (insert-file-contents-literally filename)
          (setq default-directory "/no/such/dir/exists/")
          (format-decode-buffer 'tcd)
          (kill-buffer nil)
          (setq count (1+ count))))

      (message "  %d tests" count)
      count)))


